@@ -5,7 +5,7 @@
 
 #ifndef lint
 #ifdef sccs
-static char sccsid[] = "@(#)array.c 1.16 92/10/29";
+static char sccsid[] = "@(#)array.c 1.17 93/10/25";
 #endif
 #endif
 
@@ -127,7 +127,9 @@ array_tile_init(parent, array_tile, avlist)
 	FLAG_SET(ainfo->aflags,	AF_AUTO_POSITION);
 	array_tile_reset_dimensions(ainfo, 4, 1);
 	calc_array_rect_size(ainfo, &rinfo->rect);
-	rinfo->rectobj_ops = (Rectobj_ops*)&rectobj_ops;
+
+	rinfo->rectobj_ops = rectobj_ops_find(&rectobj_ops);
+	rinfo->rectobj_ops->ref_count++;
 
 	return(XV_OK);
 }
