@@ -5,7 +5,7 @@
 
 #ifndef lint
 #ifdef sccs
-static char     sccsid[] = "@(#)canshell.c 1.50 92/11/08";
+static char     sccsid[] = "@(#)canshell.c 1.53 93/10/25";
 #endif
 #endif
 
@@ -73,7 +73,8 @@ canvas_shell_init(parent, canvas_shell, avlist)
 	 * event in repaint proc.  xv_set can override.
 	 */
 	RF_UNSET(&csinfo->rectobj_info, RF_PAINTED);
-	csinfo->rectobj_info.rectobj_ops = (Rectobj_ops*) &rectobj_ops;
+	csinfo->rectobj_info.rectobj_ops = rectobj_ops_find(&rectobj_ops);
+	csinfo->rectobj_info.rectobj_ops->ref_count++;
 
 	xv_set(canvas_shell, 
 		OPENWIN_SPLIT,
@@ -262,8 +263,8 @@ canvas_shell_set_avlist(canvas_shell, avlist_arg)
 		if(!no_pixmap_batching) {
 			csinfo->batch_repaint = (char)*(avlist+1);
 			canvas_shell_alloc_batch_pixmap(canvas_shell, csinfo,
-				xv_get(canvas_shell, XV_WIDTH), 
-				xv_get(canvas_shell, XV_HEIGHT), 
+				xv_get(canvas_shell, CANVAS_WIDTH), 
+				xv_get(canvas_shell, CANVAS_HEIGHT), 
 				xv_get(canvas_shell, WIN_DEPTH)); 
 		}
 		avlist = attr_next(avlist);
@@ -284,9 +285,22 @@ canvas_shell_set_avlist(canvas_shell, avlist_arg)
 			(RF_STATE_CREATED|RF_REPAINT|RF_CLEAR));
 		csinfo->rectobj_info.invocation_level = 1;
 		csinfo->rectobj_info.rect.r_width = 
-				(short) xv_get(canvas_shell, XV_WIDTH);
+				(short) xv_get(canvas_shell, CANVAS_WIDTH);
 		csinfo->rectobj_info.rect.r_height = 
-				(short) xv_get(canvas_shell, XV_HEIGHT);
+				(short) xv_get(canvas_shell, CANVAS_HEIGHT);
+
+		/*
+		 * Allocating the batch pixmap here is necessary because on 
+		 * init the sizes are zero.  Otherwise if the attribute is set 
+		 * to TRUE in the create attrlist, the pixmap is not
+		 * created.
+		 */
+		if(!no_pixmap_batching && csinfo->batch_repaint)
+			canvas_shell_alloc_batch_pixmap(canvas_shell, csinfo,
+				csinfo->rectobj_info.rect.r_width,
+				csinfo->rectobj_info.rect.r_height,
+				xv_get(canvas_shell, WIN_DEPTH)); 
+
 		if(csinfo->auto_drop_site) {
 			canvas_shell_create_drop_site(xv_get(canvas_shell, 
 				CANVAS_NTH_PAINT_WINDOW, 0));
@@ -928,10 +942,12 @@ canvas_shell_resize_proc(canvas_shell, width, height)
 	if(csinfo->resize_proc)
 		(csinfo->resize_proc)(canvas_shell, width, height);
 	recursion_guard = FALSE;
-	rinfo->rect.r_width = width;
-	rinfo->rect.r_height = height;
+	rinfo->rect.r_width = xv_get(canvas_shell, CANVAS_WIDTH);
+	rinfo->rect.r_height = xv_get(canvas_shell, CANVAS_HEIGHT);
 
-	canvas_shell_alloc_batch_pixmap(canvas_shell, csinfo, width, height);
+	canvas_shell_alloc_batch_pixmap(canvas_shell, csinfo,
+			xv_get(canvas_shell, CANVAS_WIDTH),
+			xv_get(canvas_shell, CANVAS_HEIGHT));
 	if(csinfo->auto_drop_site)
 		canvas_shell_set_drop_region(canvas_shell, &rinfo->rect);
 
@@ -990,9 +1006,13 @@ rectobj_set_event_grab(canvas_shell, grab_rectobj, grab_event_proc, grab_arg)
 	Proc_ptr	grab_event_proc;
 	void		*grab_arg;
 {
-	Canvas_shell_info	*csinfo = CANVAS_SHELL_PRIVATE(canvas_shell);
+	Canvas_shell_info	*csinfo;
 	Rectobj_info		*rinfo;
 
+	if(!canvas_shell)
+		return;
+
+	csinfo = CANVAS_SHELL_PRIVATE(canvas_shell);
 	csinfo->grab_event_proc = grab_event_proc;
 	csinfo->grab_arg = grab_arg;
 	csinfo->grab_rectobj = grab_rectobj;
@@ -1019,7 +1039,7 @@ canvas_shell_set_drop_region(canvas_shell, r)
 	CANVAS_EACH_PAINT_WINDOW(canvas_shell, paint_window)
 
 		drop_site = (Xv_drop_site) xv_get(paint_window, 
-				CANVAS_SHELL_DROP_SITE_KEY);
+				CANVAS_SHELL_DROP_SITE);
 
 		if(drop_site)
 		  xv_set(drop_site,
@@ -1043,7 +1063,7 @@ canvas_shell_create_drop_site(paint_window)
 		DROP_SITE_EVENT_MASK, (DND_ENTERLEAVE|DND_MOTION),
 		NULL);
 	xv_set(paint_window,
-		CANVAS_SHELL_DROP_SITE_KEY, drop_site,
+		CANVAS_SHELL_DROP_SITE, drop_site,
 		NULL);
 }
 
