@@ -6,7 +6,7 @@
 
 #ifndef lint
 #ifdef sccs
-static  char sccsid[] = "@(#) display_list.c 1.43 92/10/20 ";
+static  char sccsid[] = "@(#) display_list.c 1.45 93/03/26 ";
 #endif
 #endif
 
@@ -483,7 +483,7 @@ Dmapimage(dl_arg, data, event)
 		return FALSE;
 
 	height = (int) xv_get(data->image, XV_HEIGHT);
-	if(y+height < event_x(event))
+	if(y+height < event_y(event))
 		return FALSE;
 
 	return TRUE;
@@ -692,6 +692,9 @@ update_gc(dl_arg, data)
 		short color;
 		Rectobj_info *rinfo = RECTOBJ_PRIVATE(dl_arg->drawarea);
 
+		if(dl_arg->cms == NULL)
+			break;
+
 		if(rinfo->shared_info->cms == dl_arg->cms) {
 			color = data->data.color;
 			if(color >= rinfo->shared_info->num_colors)
@@ -711,6 +714,9 @@ update_gc(dl_arg, data)
 		short color;
 		Rectobj_info *rinfo = RECTOBJ_PRIVATE(dl_arg->drawarea);
 
+		if(dl_arg->cms == NULL)
+			break;
+
 		if(rinfo->shared_info->cms == dl_arg->cms) {
 			color = data->data.color;
 			if(color >= rinfo->shared_info->num_colors)
@@ -758,7 +764,12 @@ update_gc(dl_arg, data)
 		break;
 
 	case Set_Cms:
-		dl_arg->cms = data->data.cms;
+		if(data->data.cms != NULL)
+			dl_arg->cms = data->data.cms;
+		else {
+			Rectobj_info *rinfo = RECTOBJ_PRIVATE(dl_arg->drawarea);
+			dl_arg->cms = rinfo->shared_info->cms;
+		}
 		break;
 	}
 }
