@@ -5,7 +5,7 @@
 
 #ifndef lint
 #ifdef sccs
-static char     sccsid[] = "@(#)rectobj.c 1.80 92/11/08";
+static char     sccsid[] = "@(#)rectobj.c 1.86 93/11/04";
 #endif
 #endif
 
@@ -17,8 +17,7 @@ static char     sccsid[] = "@(#)rectobj.c 1.80 92/11/08";
 #include <limits.h>
 #include "rectobj_impl.h"
 
-static char *sspkg_patchlevel = 
-"SlingShot extensions SSPKG_VERSION (Patch level SSPKG_PATCHLEVEL)";
+static char *sspkg_patchlevel = "SlingShot extensions 2.1";
 
 Pkg_private int 	rectobj_init();
 Pkg_private Xv_opaque	rectobj_set_avlist();
@@ -27,6 +26,8 @@ Pkg_private int 	rectobj_destroy();
 
 static	void	rectobj_set_class_func();
 static	void	rectobj_add_to_parent_list(); 
+static	void	rectobj_set_highlight_group();
+static  Rectobj_ops *rectobj_ops_find_diff();
 
 int	rectobj_global_invocation_level = 0;
 
@@ -73,7 +74,8 @@ rectobj_init(parent, rectobj, avlist)
 	rectobj_object->private_data = (Xv_opaque) rinfo;
 
 	rectobj_internal_init(rectobj, rinfo);
-	rinfo->rectobj_ops = (Rectobj_ops*)&rectobj_ops;
+	rinfo->rectobj_ops = rectobj_ops_find(&rectobj_ops);
+	rinfo->rectobj_ops->ref_count++;
 	rinfo->parent = parent;
 
 	if(parent) {
@@ -187,11 +189,16 @@ rectobj_set_avlist(rectobj, avlist)
 			break;
 
 		case XV_SHOW:
-			if(*avlist++)
-				RF_SET(rinfo, RF_PAINTED);
-			else
+			if(*avlist++) {
+				RF_SET(rinfo, (RF_PAINTED|RF_REPAINT|RF_CLEAR));
+			} else {
+				/* Repaint here because rectobj_repaint_rect
+				 * won't repaint when the object is not
+				 * visible.
+				 */
+				rectobj_repaint_rect(rectobj, NULL, TRUE);
 				RF_UNSET(rinfo, RF_PAINTED);
-			RF_SET(rinfo, (RF_REPAINT|RF_CLEAR));
+			}
 			break;
 
 		case RECTOBJ_PARENT:
@@ -219,7 +226,7 @@ rectobj_set_avlist(rectobj, avlist)
 		case RECTOBJ_SELECTED:
 			if(*avlist++) {
 				rectobj_add_to_selected_list(rectobj, 
-					FALSE, NULL);
+					RF_IS_SET(rinfo, RF_EXCLUSIVE_SELECT), NULL);
 				rectobj_set_paint_style(rectobj, NULL, 
 					RECTOBJ_HIGHLIGHT);
 			} else {
@@ -342,6 +349,27 @@ rectobj_set_avlist(rectobj, avlist)
 			rectobj_repaint_rect(rectobj, NULL, TRUE);
 			break;
 
+		case RECTOBJ_HIGHLIGHT_GROUP: 
+			/* 
+			 * Object's children share state regarding
+			 * display states, exclusive or non-exclusive.
+			 * This should probably be create only or
+			 * when object has no children.
+			 */
+			if(*avlist++)
+				RF_SET(rinfo, RF_HIGHLIGHT_GROUP);
+			else
+				RF_UNSET(rinfo, RF_HIGHLIGHT_GROUP);
+			break;
+
+		case RECTOBJ_EVENT_GROUP:
+			if(*avlist++) {
+				RF_SET(rinfo, RF_EVENT_GROUP);
+			} else {
+				RF_UNSET(rinfo, RF_EVENT_GROUP);
+			}
+			break;
+
 		case XV_END_CREATE:
 			RF_UNSET(rinfo, RF_STATE_INIT);
 			RF_SET(rinfo, (RF_STATE_CREATED|RF_REPAINT|RF_CLEAR));
@@ -659,6 +687,15 @@ rectobj_get_attr(rectobj, status, attr, avlist)
 		case RECTOBJ_RESTACK_CHILDREN:
 			return (Xv_opaque) (RF_TRUE(rinfo, RF_RESTACK_CHILDREN));
 
+		case RECTOBJ_HIGHLIGHT_GROUP:
+			return (Xv_opaque) (RF_TRUE(rinfo, RF_HIGHLIGHT_GROUP));
+
+		case RECTOBJ_HIGHLIGHT_LIST:
+			return (Xv_opaque) rectobj_get_selected_list2(rectobj);
+
+		case RECTOBJ_EVENT_GROUP:
+			return (Xv_opaque) (RF_TRUE(rinfo, RF_EVENT_GROUP));
+
 		case XV_SHOW:
 			return (Xv_opaque) (RF_TRUE(rinfo, RF_PAINTED));
 
@@ -694,7 +731,7 @@ rectobj_destroy(rectobj, status)
 	RF_SET(rinfo, RF_STATE_DESTROYING);
 	RF_UNSET(rinfo, RF_STATE_CREATED);
 	
-	if(RF_IS_SET(rinfo, RF_HAS_EVENT_GRAB))
+	if(canvas_shell && RF_IS_SET(rinfo, RF_HAS_EVENT_GRAB))
 		rectobj_set_event_grab(canvas_shell, rectobj, 0, 0);
 
 	rectobj_destroy_children(rectobj);
@@ -802,9 +839,8 @@ rectobj_set_class_func(rinfo, which, fn)
 		 * Shared rectobj_ops:  copy and set private to this rectobj.
 		 */
 		Rectobj_ops	*tmp;
-		tmp = (Rectobj_ops*) malloc(sizeof(Rectobj_ops));
-		*tmp = *(rinfo->rectobj_ops);
-		tmp->ref_count = 1;
+		tmp = rectobj_ops_find_diff(rinfo->rectobj_ops, which, fn);
+		tmp->ref_count++;
 		rinfo->rectobj_ops = tmp;
 	}
 
@@ -1356,3 +1392,134 @@ rectobj_del_child_proc(parent, child, old_rect)
 
 #endif
 
+
+  
+/****************************************************************************
+ *  Automatically shared RECTOBJ_OPS tables.
+ *
+ *  J. Michael Lake, NEC Research Institute, Princeton, NJ  08540
+ *  jmlake@research.nj.nec.com  (jmlake@uiuc.edu after 20 August 1993)
+ *  03 August 1993
+ ****************************************************************************/
+
+typedef struct _o_h_e {
+	int		pre_hash;
+	struct _o_h_e	*next;
+	Rectobj_ops 	*r;
+} Rectobj_ops_list;
+
+#define TABLE_SIZE         (101)
+#define SELECTION_PROC_IDX (RECTOBJ_PROCS_ATTR_INDEX(RECTOBJ_SELECTION_PROC))
+
+static Rectobj_ops_list	*hash_table_head[TABLE_SIZE] = { NULL };
+
+static      int          rectobj_ops_diff_hash();
+Pkg_private Rectobj_ops *rectobj_ops_find     ();
+
+static int
+rectobj_ops_diff_hash(r, func, func_value)
+	Rectobj_ops	*r;
+	int		func;
+	Proc_ptr	func_value;
+{
+	int rv;
+
+	switch (func) {
+	case RECTOBJ_PROCS_ATTR_INDEX(RECTOBJ_EVENT_PROC)	:
+		rv = ( (  ((int)r->manage_child_proc  )
+			+ ((int)func_value            )
+			+ ((int)r->single_click_proc  )
+			+ ((int)r->dbl_click_proc     )
+			+ ((int)r->paint_proc         )) >> 4);
+		break;
+	case RECTOBJ_PROCS_ATTR_INDEX(RECTOBJ_MANAGE_CHILD_PROC):
+		rv = ( (  ((int)func_value            )
+			+ ((int)r->event_proc         )
+			+ ((int)r->single_click_proc  )
+			+ ((int)r->dbl_click_proc     )
+			+ ((int)r->paint_proc         )) >> 4);
+		break;
+	case RECTOBJ_PROCS_ATTR_INDEX(RECTOBJ_SINGLE_CLICK_PROC):
+		rv = ( (  ((int)r->manage_child_proc  )
+			+ ((int)r->event_proc         )
+			+ ((int)func_value            )
+			+ ((int)r->dbl_click_proc     )
+			+ ((int)r->paint_proc         )) >> 4);
+		break;
+	case RECTOBJ_PROCS_ATTR_INDEX(RECTOBJ_DBL_CLICK_PROC)	:
+		rv = ( (  ((int)r->manage_child_proc  )
+			+ ((int)r->event_proc         )
+			+ ((int)r->single_click_proc  )
+			+ ((int)func_value            )
+			+ ((int)r->paint_proc         )) >> 4);
+		break;
+	case RECTOBJ_PROCS_ATTR_INDEX(RECTOBJ_PAINT_PROC)	:
+		rv = ( (  ((int)r->manage_child_proc  )
+			+ ((int)r->event_proc         )
+			+ ((int)r->single_click_proc  )
+			+ ((int)r->dbl_click_proc     )
+			+ ((int)func_value            )) >> 4);
+		break;
+	default:
+		rv = ( (  ((int)r->manage_child_proc  )
+			+ ((int)r->event_proc         )
+			+ ((int)r->single_click_proc  )
+			+ ((int)r->dbl_click_proc     )
+			+ ((int)r->paint_proc         )) >> 4);
+	}
+
+	return (0x0000ffff & rv);
+}
+
+
+Pkg_private Rectobj_ops *
+rectobj_ops_find(r)
+	Rectobj_ops	*r;
+{
+	/* a convenience routine, used in <subclass>_init() only */
+	return rectobj_ops_find_diff(r, 0, r->paint_proc);
+}
+
+static Rectobj_ops *
+rectobj_ops_find_diff(r, func, func_value)
+	Rectobj_ops		*r;
+	int			func;
+	Proc_ptr		func_value;
+{
+	int			pre_hash, hash, i, found ;
+	Rectobj_ops_list	*temp;
+	Rectobj_ops_array 	*ra = (Rectobj_ops_array*)r;
+
+	pre_hash = rectobj_ops_diff_hash(r, func, func_value);
+	hash     = pre_hash % TABLE_SIZE;
+	temp     = hash_table_head[ hash ];
+	for(found = 0; !found && temp; temp = temp->next) {
+		Rectobj_ops_array *t = (Rectobj_ops_array *)temp->r;
+
+		if (temp->pre_hash != pre_hash  ) continue;
+		if (t->proc[func]  != func_value) continue;
+		found = 1;
+		for(i = 0; found && (i <= SELECTION_PROC_IDX); ++i) {
+			if (func == i                ) continue;
+			if (ra->proc[i] != t->proc[i]) found = 0;
+		}
+		if (found) return temp->r;
+	}
+
+	if (!(temp = (Rectobj_ops_list*)malloc(sizeof(Rectobj_ops_list)))) {
+		return NULL;	/* could not allocate space for table */
+	} else {
+		temp->pre_hash = pre_hash;
+		temp->next     = hash_table_head[ hash ];
+		if (temp->r    = (Rectobj_ops*)malloc(sizeof(Rectobj_ops))) {
+			*temp->r           = *r;  /* copy it.... */
+			temp->r->ref_count =  1;  /* claim it... */
+			((Rectobj_ops_array*)temp->r)->proc[func] = func_value;
+		}
+		hash_table_head[hash] = temp;
+	}
+	return temp->r;
+}
+
+
+
