@@ -5,7 +5,7 @@
 
 #ifndef lint
 #ifdef sccs
-static char sccsid[] = "@(#)tree.c 1.20 92/11/08";
+static char sccsid[] = "@(#)tree.c 1.21 93/10/25";
 #endif
 #endif
 
@@ -44,6 +44,7 @@ tree_init(parent, tree, avlist)
 	Tree		tree;
 	Attr_avlist	avlist;
 {
+	Rectobj_info	*rinfo = RECTOBJ_PRIVATE(tree);
 	Tree_info	*tinfo;
 	Tree_struct	*tree_object;
 	static Rectobj_ops rectobj_ops = {
@@ -61,7 +62,8 @@ tree_init(parent, tree, avlist)
 	tree_object = (Tree_struct*) tree;
 	tree_object->private_data = (Xv_opaque) tinfo;
 
-	RECTOBJ_PRIVATE(tree)->rectobj_ops = (Rectobj_ops*)&rectobj_ops;
+	rinfo->rectobj_ops = rectobj_ops_find(&rectobj_ops);
+	rinfo->rectobj_ops->ref_count++;
 
 	tinfo->parent_distance = 30;
 	tinfo->layout = TREE_LAYOUT_HORIZONTAL;
